@@ -7,11 +7,8 @@
 
 
 class BaseFilter:
+    """ Base class for filter object
     """
-    过滤器基类
-    """
-    # 错误码
-    error_code = 500
 
     def __init__(self, key, value, rule):
         """
@@ -30,10 +27,5 @@ class BaseFilter:
         """
         return True
 
-    def fmt_error_message(self, code):  # pylint: disable=unused-argument
-        """ 返回格式化的错误消息
-        """
-        return None
-
     def __call__(self, *args, **kwargs):
         pass
