@@ -5,17 +5,22 @@
 # @Author: 'Wu Dong <wudong@eastwu.cn>'
 # @Time: '2020-04-01 09:47'
 # sys
+import typing as t
 from functools import wraps
 from inspect import isfunction
 from inspect import getfullargspec
 # 3p
-from flask import g
+from flask import (  # pylint: disable=unused-import
+    Flask,
+    g,
+    request,
+)
 from werkzeug.datastructures import FileStorage
 # object
 from .exception import ParamsValueError
-from .filters.base import BaseFilter
+from .filters.base import BaseFilter  # pylint: disable=unused-import
 from .filters import (
-    complex_filters,
+    cross_filters,
     simple_filters,
 )
 from .macro import (
@@ -30,15 +35,27 @@ from .response import (
     JSONResponse,
 )
 from .rules import Rule
-from .utils import get_deep_value
-from . import filters
+from .utils import (
+    get_deep_value,
+    missing
+)
+# checking
+if t.TYPE_CHECKING:
+    from flask import Response  # pylint: disable=unused-import
 
 
 class PreRequest:
     """ An object to dispatch filters to handler request params
     """
 
-    def __init__(self, app=None, fuzzy=False, store_key=None, content_type=None, skip_filter=False):
+    def __init__(
+            self,
+            app: t.Optional["Flask"] = None,
+            fuzzy: bool = False,
+            store_key: t.Optional[str] = None,
+            content_type: t.Optional[str] = None,
+            skip_filter: bool = False
+    ):
         """ PreRequest init function
 
         :param fuzzy: formatter error message with fuzzy style
@@ -46,20 +63,21 @@ class PreRequest:
         :param content_type: response content type json/html
         :param skip_filter: skip all of the filter check
         """
-        self.filters = simple_filters
-        self.complex_filters = complex_filters
-        self.fuzzy = fuzzy
-        self.content_type = content_type or "application/json"
-        self.store_key = store_key or "params"
-        self.response = None
-        self.formatter = None
-        self.skip_filter = skip_filter
+        self.simple_filters: t.List["BaseFilter"] = simple_filters
+        self.cross_filters: t.List["BaseFilter"] = cross_filters
+
+        self.fuzzy: bool = fuzzy
+        self.content_type: str = content_type or "application/json"
+        self.store_key: str = store_key or "params"
+        self.response: t.Optional[BaseResponse] = None
+        self.formatter: t.Optional[t.Callable] = None
+        self.skip_filter: bool = skip_filter
 
         if app is not None:
-            self.app = app
+            self.app: "Flask" = app
             self.init_app(app, None)
 
-    def init_app(self, app, config=None):
+    def init_app(self, app: "Flask", config: t.Optional[dict] = None):
         """ Flask extension initialize
 
         :param app: flask application
@@ -82,17 +100,14 @@ class PreRequest:
         self.app = app
         app.extensions["pre_request"] = self
 
-    def add_response(self, resp):
+    def add_response(self, resp: BaseResponse):
         """ Add custom response class
 
         :param resp: response class which is subclass of BaseResponse
         """
-        if resp and not issubclass(resp, BaseResponse):
-            raise TypeError("custom response must be subclass of `pre_request.BaseResponse`")
-
         self.response = resp
 
-    def add_formatter(self, fmt):
+    def add_formatter(self, fmt: t.Callable):
         """ Add custom format function for generate response content
 
         :param fmt: format function
@@ -100,39 +115,36 @@ class PreRequest:
         if fmt and not isfunction(fmt):
             raise TypeError("custom format function must be a type of function")
 
-        if fmt and fmt.__code__.co_argcount < 2:
-            raise TypeError("custom format function requires at least 2 arguments")
+        if fmt and fmt.__code__.co_argcount < 1:
+            raise TypeError("custom format function requires at least 1 arguments")
 
         self.formatter = fmt
 
-    def add_filter(self, cus_filter, index=None):
+    def add_filter(self, cus_filter: "BaseFilter", index: t.Optional[int] = None):
         """ Add custom filter class to extend pre-request
 
         :param cus_filter: custom filter class
         :param index: filter position
         """
-        if cus_filter and not issubclass(cus_filter, BaseFilter):
-            raise TypeError("custom filter must be subclass of `BaseFilter`")
-
         if index is not None and not isinstance(index, int):
             raise TypeError("index params must be type of Int")
 
         if index is not None:
-            self.filters.insert(index, cus_filter)
+            self.simple_filters.insert(index, cus_filter)
         else:
-            self.filters.append(cus_filter)
+            self.simple_filters.append(cus_filter)
 
-    def remove_filter(self, cus_filter=None, index=None):
+    def remove_filter(self, cus_filter: t.Optional["BaseFilter"] = None, index: t.Optional[int] = None):
         """ Remove filters from object with index or filter name
 
         :param cus_filter: user filter name
         :param index: filter index
         """
-        if cus_filter and (isinstance(cus_filter, str) or issubclass(cus_filter, BaseFilter)):
-            self.filters.remove(cus_filter)
+        if cus_filter:
+            self.simple_filters.remove(cus_filter)
 
-        if index is not None and isinstance(index, int) and 0 <= index < len(self.filters):
-            self.filters.pop(index)
+        if index is not None and isinstance(index, int) and 0 <= index < len(self.simple_filters):
+            self.simple_filters.pop(index)
 
     @staticmethod
     def _location_params(key, location, default=None, deep=True):
@@ -143,8 +155,6 @@ class PreRequest:
         :param default: default value if special value is not exists
         :param deep: read params with deep search
         """
-        from flask import request  # pylint: disable=import-outside-toplevel
-
         location = location.lower()
 
         if location in ["args", "values", "form", "headers", "cookies"]:
@@ -171,11 +181,10 @@ class PreRequest:
         :param key: params key
         """
         df_location = ["values", "args", "form", "json", "headers", "cookies"]
-        from flask import request  # pylint: disable=import-outside-toplevel
 
         if len(key.split(".")) > 1 and rule.deep:
             rst = get_deep_value(key, getattr(request, "json"), default, deep=True)
-            # load value from depth json structure failed
+            # load value from depth json struct failed
             if rst != default:
                 return rst
 
@@ -197,14 +206,12 @@ class PreRequest:
         :param key: params key
         :param rule: params rule
         """
-        from flask import request  # pylint: disable=import-outside-toplevel
-
         # load single params
         if not rule.multi:
             return request.files.get(key)
 
         # load multi files
-        fmt_params = list()
+        fmt_params = []
         for f in request.files.getlist(key):
             fmt_params.append(f)
         return fmt_params
@@ -216,7 +223,7 @@ class PreRequest:
         :param r: params rule
         """
         if isinstance(r, dict):
-            fmt_result = dict()
+            fmt_result = {}
             for key, rule in r.items():
                 fmt_value = self._handler_simple_filter(k + "." + key, v, rule)
                 fmt_result[rule.key_map if isinstance(rule, Rule) and rule.key_map else key] = fmt_value
@@ -224,7 +231,7 @@ class PreRequest:
             return fmt_result
 
         if not isinstance(r, Rule):
-            raise TypeError("invalid rule type for key '%s'" % k)
+            raise TypeError(f"invalid rule type for key '{k}'")
 
         if v is None:
             # load file type of params from request
@@ -233,34 +240,34 @@ class PreRequest:
 
             # load simple params
             else:
-                v = self._fmt_params(k, r)
+                v = self._fmt_params(k, r, default=missing)
 
-        if r.structure is not None:
+        if r.struct is not None:
             # make sure that input value is not empty
             if r.required and not v:
-                raise ParamsValueError(560, message="%s field cannot be empty" % k)
+                raise ParamsValueError(message=f"{k} field cannot be empty")
 
             if not r.multi:
-                raise TypeError("invalid usage of `structure` params")
+                raise TypeError("invalid usage of `struct` params")
 
-            # structure params must be type of list
+            # struct params must be type of list
             if not isinstance(v, list):
-                raise ParamsValueError(601, message="Input " + k + " invalid type")
+                raise ParamsValueError(message="Input " + k + " invalid type")
 
             if not v:
-                return list()
+                return []
 
             # storage sub array
-            fmt_result = list()
+            fmt_result = []
             for idx, sub_v in enumerate(v):
                 # make sure that array item must be type of dict
                 if not isinstance(sub_v, dict):
-                    raise ParamsValueError(600, message="Input " + k + "." + str(idx) + " invalid type")
+                    raise ParamsValueError(message="Input " + k + "." + str(idx) + " invalid type")
 
-                # format every k-v with structure
-                fmt_item = dict()
+                # format every k-v with struct
+                fmt_item = {}
                 fmt_result.append(fmt_item)
-                for sub_k, sub_r in r.structure.items():
+                for sub_k, sub_r in r.struct.items():
                     new_k = k + "." + str(idx) + "." + sub_k
                     v = self._handler_simple_filter(new_k, sub_v.get(sub_k), sub_r)
                     fmt_item[sub_r.key_map if isinstance(sub_r, Rule) and sub_r.key_map else sub_k] = v
@@ -271,18 +278,11 @@ class PreRequest:
             return v
 
         # filter request params
-        for f in self.filters:
-            filter_obj = None
-            # system filter object
-            if isinstance(f, str):
-                filter_obj = getattr(filters, f)(k, v, r)
-
-            # custom filter object
-            elif issubclass(f, BaseFilter):
-                filter_obj = f(k, v, r)
+        for f in self.simple_filters:
+            filter_obj = f(k, v, r)
 
             # ignore invalid and not required filter
-            if not filter_obj or not filter_obj.filter_required():
+            if not filter_obj.filter_required():
                 continue
 
             v = filter_obj()
@@ -292,53 +292,48 @@ class PreRequest:
 
         return v
 
-    def _handler_complex_filter(self, k, r, rst):
+    def _handler_cross_filter(self, k, r, rst):
         """ Handler complex rule filters
 
         :param k: params key
         :param r: params rule
         :param rst: handler result
-        :param rules: request rules
         """
         if isinstance(r, dict):
             for key, value in r.items():
-                self._handler_complex_filter(k + "." + key, value, rst)
+                self._handler_cross_filter(k + "." + key, value, rst)
             return
 
         if not isinstance(r, Rule):
-            raise TypeError("invalid rule type for key '%s'" % k)
+            raise TypeError(f"invalid rule type for key '{k}'")
 
         if r.skip or self.skip_filter:
             return
 
         # simple filter handler
-        for f in self.complex_filters:
-            filter_obj = None
-            # system filter object
-            if isinstance(f, str):
-                filter_obj = getattr(filters, f)(k, None, r)
-
-            # custom filter object
-            elif issubclass(f, BaseFilter):
-                filter_obj = f(k, None, r)
+        for f in self.cross_filters:
+            filter_obj = f(k, None, r)
 
             # ignore invalid and not required filter
-            if not filter_obj or not filter_obj.filter_required():
+            if not filter_obj.filter_required():
                 continue
 
             filter_obj(params=rst)
 
-    def parse(self, rule=None, **options):
+    def parse(
+            self,
+            rule: t.Optional[t.Dict[str, t.Union["Rule", dict]]] = None,
+            **options
+    ) -> dict:
         """ Parse input params
         """
-        fmt_rst = dict()
+        fmt_rst = {}
 
         # invalid input
         if not rule and not options:
             return fmt_rst
 
         # query rules with special method
-        from flask import request  # pylint: disable=import-outside-toplevel
         rules = options.get(request.method) or options.get(request.method.lower())
 
         # common rule
@@ -347,7 +342,7 @@ class PreRequest:
 
         # ignore catch with empty rules
         if not rules:
-            raise ValueError("request method '%s' with invalid filter rule" % request.method)
+            raise ValueError(f"request method '{request.method}' with invalid filter rule")
 
         # use simple filter to handler params
         for k, r in rules.items():
@@ -357,14 +352,18 @@ class PreRequest:
 
         # use complex filter to handler params
         for k, r in rules.items():
-            self._handler_complex_filter(k, r, fmt_rst)
+            self._handler_cross_filter(k, r, fmt_rst)
 
         return fmt_rst
 
-    def catch(self, rule=None, **options):
+    def catch(
+            self,
+            rule: t.Optional[t.Dict[str, t.Union["Rule", dict]]] = None,
+            **options
+    ) -> t.Callable:
         """ Catch request params
         """
-        def decorator(func):
+        def decorator(func: t.Callable) -> t.Callable:
             @wraps(func)
             def wrapper(*args, **kwargs):
                 # ignore with empty rule
@@ -386,15 +385,15 @@ class PreRequest:
             return wrapper
         return decorator
 
-    def fmt_resp(self, error):
+    def fmt_resp(self, error: ParamsValueError) -> "Response":
         """ Handler not formatted request error
 
         :param error: ParamsValueError
         """
         if self.response is not None:
-            return self.response()(self.fuzzy, self.formatter, error)
+            return self.response.make_response(error, self.fuzzy, self.formatter)
 
         if self.content_type == "text/html":
-            return HTMLResponse()(self.fuzzy, self.formatter, error)
+            return HTMLResponse.make_response(error, self.fuzzy, self.formatter)
 
-        return JSONResponse()(self.fuzzy, self.formatter, error)
+        return JSONResponse.make_response(error, self.fuzzy, self.formatter)
